#!/usr/bin/perl

use strict;
use warnings;
use LWP::UserAgent;
use WWW::Mechanize;
use HTTP::Response;
use HTTP::Cookies;

# $Id: facebook.pl 139 2008-05-09 22:24:30Z ed $
# $URL: svn+ssh://fw/repos/code/perl/facebook/facebook.pl $

my %config;
my %albums;

sub file_contents {

}

sub set_config {
	$config{'www'} = WWW::Mechanize->new( agent=> "Mozilla" );
	open( F, "<config" )
		or die( "cannot open config file" );
	while( <F> ) {
		chomp;
		next if( $_ =~ /^#/ );

		if( $_ =~ /^(.*?)\s+=\s+(.*)$/ ) {
			$config{$1} = $2;
		}
	}
	close(F);
}

sub debug {
	my $status = shift;
	my $text = shift;

	print( STDERR $text, "\n" );
}

sub login {
	debug( 1, "getting login page" );
	my $resp = $config{'www'}->get( $config{'facebook'} );

	if( !$resp->is_success ) {
		debug( 0, $resp->status_line );
	}

	$resp = $config{'www'}->submit_form( form_name => 'loginform',
		fields => { 
			email => $config{'email'},
			pass => $config{'pass'} },
		button => "doquicklogin" 
	);
	# $config{'www'}->submit();		
	
	if( !$resp->is_success ) {
		debug( 0, $resp->status_line );
		return( undef );
	}
}

sub get_album_urls {
	debug( 0, "Getting album urls" );

	#if( !$resp->is_success ) {
	#	debug( 0, $resp->status_line );
	#	return;
	#}
	

	for( my $resp, my $last_url = undef  ;; ) {
		if( !defined( $last_url ) ) {
			$resp = $config{'www'}->get( $config{'photo_list_url'} );
			$last_url = 1;
		}
		else {
			# debug( 0, "Following link" );
			$resp = $config{'www'}->follow_link( text_regex => qr/Next/ );
		}

		last if( not defined( $resp ));
		last if( !$resp->is_success );

		debug( 0, "got link ok" );

		# print( $resp->content, "\n" );

		my @lines = split( /\n/, $resp->content );

		foreach( @lines ) {
			if( $_ =~ /$config{'album_url'}/ ) {
				my $aid = $1;
				my $id = $2;
				my $name = $3;
				$name = $4 if defined( $4 );
					
				$albums{$name} = { aid => $aid, id => $id };
			}
		}
	}
}

sub get_picture_links_from_album {
	my $album_name = shift;

	my $resp;

	for( my $last_url ;; ) {
		my $aid = $albums{$album_name}{"aid"};
		my $id = $albums{$album_name}{"id"};

		if( !defined( $last_url ) ) {
			$resp = $config{'www'}->get( "http://www.facebook.com/album.php?aid=${aid}&id=${id}" );
			$last_url = 1;
		}
		else {
			$resp = $config{'www'}->follow_link( text_regex => qr/next/ );
		}

		last if( !defined( $resp ) );
		last if( !$resp->is_success );

		# we have to use a really stupid word here to split things on since fb don't break their pages up
		my @lines = split( /img/, $resp->content );
		
		foreach( @lines ) {
			if( $_ =~ m!$config{'photo_url'}! ) {
				push( @{ $albums{$album_name}{'picture_urls'} }, "${1}:${2}" );

			}
		}
		
		#foreach( @{ $albums{$album_name}{'picture_urls'} } ) {
		#	debug( 0, "Collected: $_" );
		#}
	}
}

sub work_on_albums {
	foreach( sort keys( %albums ) ) {
		# my %h = $albums{$_};
		debug( 0, "Album $_ " . $albums{$_}{'aid'} );
		get_picture_links_from_album( $_ );
	}
	
}

sub put_photo {
	my $jpg = shift;
	my $album = shift;
	my $pid = shift;
	my $album_id = shift;
	my $album_name = shift;
	my $path = "";

	$album_name =~ s/[^a-zA-Z0-9-:]/_/g;

	mkdir( $album_name ) if( ! -d $album_name ); $path .= $album_name;

	my $resp = $config{'www'}->get( $jpg );
	return if( !defined( $resp ) );
	return if( !$resp->is_success );

	if( !$config{'www'}->save_content( "$path/$pid.jpg" ) ) {
		open( LOG, ">>history.log" );
		print( LOG "cannot save $path/$pid.jpg:$!" );
		close( LOG );
	}

	open( LIST, ">>list.txt" ) or return;
	print( LIST "$jpg\t$path/$pid.jpg", "\n" );
	close( LIST );
	
}

sub get_photo_ref {
	my $page = shift;

	my $resp = $config{'www'}->get( $page );
	return undef if( !defined( $resp ) );
	return undef if( !$resp->is_success );

	# split it up because fb go out of their way to make things un scrapeable

	my $c = $resp->content;
	open( F, ">/tmp/photo.html" )
		or die( "cannot write to /tmp/photo.html:$!" );
	print( F $resp->content );
	close(F);

	my @p = split( />/, $c );

	foreach( @p ) {
		if( $_ =~ m!$config{'image_ref'}! ) {
		#if( $_ =~ /"(.*?)" id="myphoto"/ ) {
			debug( 0, $1 );
			return( $1 );
		}
	}
	return( undef );
}

sub work_on_album_pictures {
	
	foreach my $album_name ( sort keys( %albums ) ) {
		debug( 3, "Working on album $album_name" );

		my @photos = @{ $albums{$album_name}{'picture_urls'} };

		foreach( @photos ) {
			my @p = split( /\:/, $_ );

			my $page = $config{'photo_page_url'};

			$page =~ s/{{{pid}}}/${p[0]}/g;
			$page =~ s/{{{id}}}/${p[1]}/g;

			my $ref = get_photo_ref( $page );
			if( !defined( $ref ) ) {
				debug( 1, "could not get a ref on $page" );
			}

			put_photo( $ref, $_, $p[0], $p[1], $album_name );
		}

	}
}

sub main {
	set_config();
	login();
	get_album_urls();
	work_on_albums();
	work_on_album_pictures();
}

main();

